Add comments, clean up white space, etc.
[fw/altos] / ao_task.c
index 4f7387e310f595fe921b6a3b3c604d2ebd934ce3..24bd319ac563b69d33dd77999a0a532763eaa592 100644 (file)
--- a/ao_task.c
+++ b/ao_task.c
@@ -3,8 +3,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,7 +29,7 @@ ao_add_task(__xdata struct ao_task * task, void (*start)(void))
 {
        uint8_t __xdata *stack;
        if (ao_num_tasks == AO_NUM_TASKS)
-               ao_panic(AO_ERROR_NO_TASK);
+               ao_panic(AO_PANIC_NO_TASK);
        ao_tasks[ao_num_tasks++] = task;
        /*
         * Construct a stack frame so that it will 'return'
@@ -74,9 +73,9 @@ ao_yield(void) _naked
                /* Push ACC first, as when restoring the context it must be restored
                 * last (it is used to set the IE register). */
                push    ACC
/* Store the IE register then disable interrupts. */
                push    _IEN0
-               clr     _EA
+               setb    _EA
                push    DPL
                push    DPH
                push    b
@@ -98,7 +97,7 @@ ao_yield(void) _naked
        if (ao_cur_task_id != AO_NO_TASK)
        {
                /* Save the current stack */
-               stack_len = SP - AO_STACK_START;
+               stack_len = SP - (AO_STACK_START - 1);
                ao_cur_task->stack_count = stack_len;
                stack_ptr = (uint8_t __data *) AO_STACK_START;
                save_ptr = (uint8_t __xdata *) ao_cur_task->stack;
@@ -107,7 +106,7 @@ ao_yield(void) _naked
        }
        
        /* Empty the stack; might as well let interrupts have the whole thing */
-       SP = AO_STACK_START;
+       SP = AO_STACK_START - 1;
 
        /* Find a task to run. If there isn't any runnable task,
         * this loop will run forever, which is just fine
@@ -123,11 +122,12 @@ ao_yield(void) _naked
 
        /* Restore the old stack */
        stack_len = ao_cur_task->stack_count;
+       SP = AO_STACK_START - 1 + stack_len;
+
        stack_ptr = (uint8_t __data *) AO_STACK_START;
        save_ptr = (uint8_t __xdata *) ao_cur_task->stack;
        while (stack_len--)
                *stack_ptr++ = *save_ptr++;
-       SP = (uint8_t) (stack_ptr - 1);
 
        _asm
                pop             _bp
@@ -162,7 +162,9 @@ ao_yield(void) _naked
 int
 ao_sleep(__xdata void *wchan)
 {
+       ao_interrupt_disable();
        ao_cur_task->wchan = wchan;
+       ao_interrupt_enable();
        ao_yield();
 }
 
@@ -179,6 +181,12 @@ ao_wakeup(__xdata void *wchan)
 void
 ao_start_scheduler(void)
 {
+       ao_timer_init();
+       ao_adc_init();
+       ao_beep_init();
+       ao_led_init();
+       ao_usb_init();
+
        ao_cur_task_id = AO_NO_TASK;
        ao_cur_task = NULL;
        ao_yield();
@@ -74,9 +73,9 @@ ao_yield(void) _naked
                 * last (it is used to set the IE register). */
                push    ACC
-               /* Store the IE register then disable interrupts. */
+               /* Store the IE register then enable interrupts. */
                push    _IEN0